<?php
/**
 * Custom functions that act independently of the theme templates
 *
 * Eventually, some of the functionality here could be replaced by core features
 *
 * @package WPCharming
 */

/**
 * Automatic theme updates notifications
 *
 */
if ( ! function_exists( 'wpcharming_updater' ) ) {
	function wpcharming_updater() {
		global $wpc_option;
		$username = trim( $wpc_option['tf_username'] );
		$api_key  = trim( $wpc_option['tf_api'] );
		if ( ! empty( $username ) && ! empty( $api_key ) ) {
			load_template( get_template_directory() . '/inc/updater/envato-theme-update.php' );

			if ( class_exists( 'Envato_Theme_Updater' ) ) {
				Envato_Theme_Updater::init( $username, $api_key, 'WPCharming' );
			}
		}
	}
	add_action( 'after_setup_theme', 'wpcharming_updater' );
}

/**
 * Get our wp_nav_menu() fallback, wp_page_menu(), to show a home link.
 *
 * @param array $args Configuration arguments.
 * @return array
 */
function wpcharming_page_menu_args( $args ) {
	$args['show_home'] = true;
	return $args;
}
add_filter( 'wp_page_menu_args', 'wpcharming_page_menu_args' );

/**
 * Adds custom classes to the array of body classes.
 *
 * @param array $classes Classes for the body element.
 * @return array
 */
function wpcharming_body_classes( $classes ) {

	global $woocommerce;
	global $post;

	if ( is_page_template( 'template-fullwidth.php' ) || is_404() ) {
		$classes[] = 'page-fullwidth';
	}

	if ( $woocommerce ) {
		$woo_layout  = get_post_meta( woocommerce_get_page_id('shop'), '_wpc_page_layout', true );
		if ( $woo_layout == 'right-sidebar' || $woo_layout == 'left-sidebar' ) {
			$classes[] = 'shop-has-sidebar';
		}
	}

	// Boxed Layout
	if ( wpcharming_option('site_boxed') || (isset($_REQUEST['boxed_layout']) && $_REQUEST['boxed_layout'] = 'enable' ) ) {
		$classes[] = 'layout-boxed';
	}

	// Onepage Homepage.
	if ( is_page() ) {
		if ( get_post_meta( $post->ID, '_wpc_is_one_page', true ) == 'on' ) {
			$classes[] = 'onepage-navigation';
		}
	}

	return $classes;
}
add_filter( 'body_class', 'wpcharming_body_classes' );

/**
 * Adds custom classes to main content.
 *
 * @param array $classes Classes for the body element.
 * @return array
 */
function wpcharming_get_layout_class ( ) {
	global $post;
	global $woocommerce;
	$classes              = 'right-sidebar';
	$page_layout_admin    = wpcharming_option('page_layout');
	$archive_layout_admin = wpcharming_option('archive_layout');
	$blog_layout_admin    = wpcharming_option('blog_layout');
	$single_shop_layout   = wpcharming_option('single_shop_layout');

	// Pages
	if ( is_page() ){
		$page_meta = get_post_meta( $post->ID, '_wpc_page_layout', true );
		$classes = $page_meta;

		if ( $page_meta || $page_meta != 'sidebar-default' ) {
			$classes = $page_meta;
		} else {
			$classes = $page_layout_admin;
		}
	}

	// Single Post
	if ( is_single() ) {
		if ( $blog_layout_admin ) {
			$classes = $blog_layout_admin;
		} else {
			$classes = 'right-sidebar';
		}
	}

	// Archive
	if ( (is_archive() || is_author()) & !is_front_page() ) {
		if ( $archive_layout_admin !== '' ){
			$classes = $archive_layout_admin;
		} else {
			$classes = 'right-sidebar';
		}
		
	}

	// Search
	if ( is_search() ) {
		if ( $archive_layout_admin !== '' ){
			$classes = $archive_layout_admin;
		} else {
			$classes = 'right-sidebar';
		}
		
	}

	// Blog Page
	if ( !is_front_page() && is_home() ) {
		if ( $blog_layout_admin ) {
			$classes = $blog_layout_admin;
		} else {
			$classes = 'right-sidebar';
		}
	}

	// WooCommerce
	if ( $woocommerce ) {
		$shop_layout_meta = get_post_meta( woocommerce_get_page_id('shop'), '_wpc_page_layout', true );
		if ( $woocommerce && is_shop() || $woocommerce && is_product_category() || $woocommerce && is_product_tag() ) {
			if ( $shop_layout_meta ) {
				$classes = $shop_layout_meta;
			} else {
				$classes = 'no-sidebar';
			}
		}
		if ( $woocommerce && is_product() ) {
			if ( $single_shop_layout ) {
				$classes = $single_shop_layout;
			} else {
				$classes = 'no-sidebar';
			}
		}
	}

	return $classes;
}


/**
 * Sets the authordata global when viewing an author archive.
 *
 * This provides backwards compatibility with
 * http://core.trac.wordpress.org/changeset/25574
 *
 * It removes the need to call the_post() and rewind_posts() in an author
 * template to print information about the author.
 *
 * @global WP_Query $wp_query WordPress Query object.
 * @return void
 */
function wpcharming_setup_author() {
	global $wp_query;

	if ( $wp_query->is_author() && isset( $wp_query->post ) ) {
		$GLOBALS['authordata'] = get_userdata( $wp_query->post->post_author );
	}
}
add_action( 'wp', 'wpcharming_setup_author' );

/**
 * Output the status of widets.
 *
 */
function wpcharming_sidebar_desc( $sidebar_id ) {

	$desc           = '';
	$column         = str_replace( 'footer-', '', $sidebar_id );
	$footer_columns = wpcharming_option('footer_columns');

	if ( $column > $footer_columns ) {
		$desc = __( 'This widget area is currently disabled. You can enable it Theme Options - Footer section.', 'base' );
	}

	return esc_html( $desc );
}

/**
 * Output top page slider for pages.
 *
 */
function wpcharming_toppage_slider() {
	global $post;
	if ( is_page ()) {
		$slider_id = get_post_meta( $post->ID, '_wpc_top_page_slider', true );
		if ( class_exists('RevSlider') ) {
			if ( $slider_id != '' && $slider_id != 'select_slider' ) {
				echo do_shortcode( '[rev_slider '. $slider_id .']' );
			}
		}
	}
}


